/* ************************************************************************** */
/*                                                                            */
/*                                                        :::      ::::::::   */
/*   philo.h                                            :+:      :+:    :+:   */
/*                                                    +:+ +:+         +:+     */
/*   By: vjacquie <vjacquie@student.42.fr>          +#+  +:+       +#+        */
/*                                                +#+#+#+#+#+   +#+           */
/*   Created: 2014/05/05 15:00:11 by jmoiroux          #+#    #+#             */
/*   Updated: 2014/05/20 18:04:44 by jmoiroux         ###   ########.fr       */
/*                                                                            */
/* ************************************************************************** */

#ifndef PHILO_H
# define PHILO_H

# include <stdio.h>
# include <time.h>
# include "libft/includes/libft.h"
# include <pthread.h>
# include <curses.h>

/*
** CORRECTION VALUES
** CAN BE CHANGE
*/

# define MAX_LIFE	20
# define EAT_T		2
# define REST_T		2
# define THINK_T	2
# define TIMEOUT	10

/*
** DON'T TOUCH FOLLOWING
** PHY_0 : height
** PHX_0 : witdh
*/

# define ESC		27
# define LEFT		10
# define RIGHT		20
# define EAT		1
# define SLEEP		2
# define THINK		3
# define DIE		4
# define WIN_Y		70
# define WIN_X		180
# define TY			25
# define TX			65
# define PHY_0		5
# define PHX_0		85
# define PHY_1		15
# define PHX_1		130
# define PHY_2		40
# define PHX_2		140
# define PHY_3		58
# define PHX_3		115
# define PHY_4		58
# define PHX_4		45
# define PHY_5		40
# define PHX_5		20
# define PHY_6		15
# define PHX_6		29
# define DANCEY		30
# define DANCEX		25

/*
** used : 1 if the rod is used
** user : who use it
** u_hand : LEFT hand or RIGHT hand
*/

typedef struct		s_rod
{
	int				used;
	int				user;
	int				u_hand;
}					t_rod;

/*
** thr_nbr : thread number
** left : left hand ( by default, rod[thr_nbr])
** right : right hand ( by default, rod[thr_nbr - 1])
** think / sleep / eat : state of the user
** start : timestamp (when the phase begun)
** is_think : someone already thinking?
** die : if he died, die = 1, else 0
*/

typedef struct		s_phil
{
	int				thr_nbr;
	int				hp;
	struct s_rod	*left;
	struct s_rod	*right;
	int				think;
	int				sleep;
	int				eat;
	time_t			start;
	int				*run;
	int				*is_think;
	int				die;
}					t_phil;

/*
** phil0_hp : health point
** phil0_state : user's state(EAT, SLEEP or THINK)
*/

typedef struct		s_data
{
	int				phil0_hp;
	int				phil0_state;
	int				phil1_hp;
	int				phil1_state;
	int				phil2_hp;
	int				phil2_state;
	int				phil3_hp;
	int				phil3_state;
	int				phil4_hp;
	int				phil4_state;
	int				phil5_hp;
	int				phil5_state;
	int				phil6_hp;
	int				phil6_state;
	int				run;
	int				is_think;
	time_t			ttimeout;
	pthread_t		thread[7];
	t_rod			rod[7];
	t_phil			phil[7];
	WINDOW			*window;
}					t_data;

void				phil0(WINDOW *window);
void				phil1(WINDOW *window);
void				phil2(WINDOW *window);
void				phil3(WINDOW *window);
void				phil4(WINDOW *window);
void				phil5(WINDOW *window);
void				phil6(WINDOW *window);
void				rod0(WINDOW *window);
void				rod0_2(WINDOW *window, int x, int y);
void				rod0_3(WINDOW *window, int x, int y);
void				rod0_4(WINDOW *window, int x, int y);
void				rod1(WINDOW *window);
void				rod1_2(WINDOW *window, int x, int y);
void				rod1_3(WINDOW *window, int x, int y);
void				rod1_4(WINDOW *window, int x, int y);
void				rod2(WINDOW *window);
void				rod2_2(WINDOW *window, int x, int y);
void				rod2_3(WINDOW *window, int x, int y);
void				rod2_4(WINDOW *window, int x, int y);
void				rod3(WINDOW *window);
void				rod3_2(WINDOW *window, int x, int y);
void				rod3_3(WINDOW *window, int x, int y);
void				rod3_4(WINDOW *window, int x, int y);
void				rod4(WINDOW *window);
void				rod4_2(WINDOW *window, int x, int y);
void				rod4_3(WINDOW *window, int x, int y);
void				rod4_4(WINDOW *window, int x, int y);
void				rod5(WINDOW *window);
void				rod5_2(WINDOW *window, int x, int y);
void				rod5_3(WINDOW *window, int x, int y);
void				rod5_4(WINDOW *window, int x, int y);
void				rod6(WINDOW *window);
void				rod6_2(WINDOW *window, int x, int y);
void				rod6_3(WINDOW *window, int x, int y);
void				rod6_4(WINDOW *window, int x, int y);
void				check_u0(t_data *d, t_phil *phil);
void				check_u1(t_data *d, t_phil *phil);
void				check_u2(t_data *d, t_phil *phil);
void				check_u3(t_data *d, t_phil *phil);
void				check_u4(t_data *d, t_phil *phil);
void				check_u5(t_data *d, t_phil *phil);
void				check_u6(t_data *d, t_phil *phil);

void				check_user(t_phil *phil);
void				init_rod(t_phil phil[]);
void				fonction(void *ptr);
void				draw_table(WINDOW *window);
void				draw_table_2(WINDOW *window);
void				ft_think(t_phil *phil);
void				ft_eat(t_phil *phil);
void				ft_sleep(t_phil *phil);
void				time_to_dance(WINDOW *window);
void				draw_print_refresh(WINDOW *window);
void				pthread_join_all(t_data *d);
void				end_game(t_data *d);

int					init_start(t_data *d);
int					pthread_create_all(t_data *d);
int					pthread_create_all_2(t_data *d);
int					prt_err(char *str);
int					init_phil(t_phil phil[],
						t_rod rod[], int *run, int *is_think);

WINDOW				*ptr_window(void);

t_data				*data(void);

#endif
